part of model_base;

@JsonSerializable(includeIfNull: false)
class Consultation extends EditableModel with _$ConsultationSerializerMixin {
  Consultation();

  factory Consultation.fromJson(Map<String, dynamic> json) =>
      _$ConsultationFromJson(json);

  bool area_face = false;
  bool area_chest = false;
  bool area_back = false;
  bool area_other = false;
  String skin_type_id;
  String salon_id;
  String service_id;
  String comments_internal;
  String description;

  @JsonKey(nullable: false)
  List<String> customer_symptoms = [];

  @JsonKey(nullable: false)
  List<String> zones = [];

  @JsonKey(nullable: false)
  List<String> product_routines = [];

  @JsonKey(nullable: false)
  List<String> image_uris = ['', '', '', ''];

  String skin_tone_id;
  String current_skin_status;

  @JsonKey(nullable: false)
  SymptomMap symptoms = new SymptomMap(
      open_comedones: 0,
      closed_comedones: 0,
      papules: 0,
      pustules: 0,
      cysts: 0,
      nodules: 0,
      seborrhea: 0,
      rosacea: 0,
      milia: false,
      pigmentation: false,
      surface_blood_vessels: false,
      acne_scars: false,
      texture: 'normal');

  /**
   * Survey data
   */
  /// Your history
  int age_started;
  String skin_progress_status;

  /// Family history
  @JsonKey(nullable: false)
  List<FamilyMember> family_members = [];

  /// Treatment history
  @JsonKey(nullable: false)
  List<UsedProduct> used_products = [];

  @JsonKey(nullable: false)
  List<UsedTreatment> used_treatments = [];

  @JsonKey(nullable: false)
  bool visited_doctor;

  @JsonKey(nullable: false)
  List<UsedProduct> used_medicines = [];

  @JsonKey(nullable: false)
  List<UsedProduct> used_supplements = [];

  /// Woman only
  String woman_monthly_when;
  @JsonKey(nullable: false)
  List<UsedProduct> used_birth_controls = [];
  String pregnancy_duration;
  String pregnancy_result;

  /// Product routine
  @JsonKey(nullable: false)
  List<String> routine_products = [];
  String routine_level;

  /// Other
  String comments;
  String call_me_time;

  /// Poke habits
  String poke_habits_frequency;
  String poke_habits_result;
  String poke_habits_consciously_other;
  String poke_habits_unconsciously_other;

  @JsonKey(nullable: false)
  List<String> poke_habits_how = [];

  @JsonKey(nullable: false)
  List<String> poke_habits_consciously = [];

  @JsonKey(nullable: false)
  List<String> poke_habits_unconsciously = [];

  /// Stress
  String stress_cause_other;

  @JsonKey(nullable: false)
  List<String> stress_causes = [];

  /// Insomnia
  int has_insomnia_hours_sleep;
  String has_insomnia_bedtime;
  String has_insomnia_time_rise;

  /// Exercise
  String does_exercise_description;

  /// Climate
  String is_climate_sensitive_description;
  String is_climate_sensitive_sun_effect;

  /// Touching objects
  @JsonKey(nullable: false)
  List<String> touching_objects = [];
  String touching_objects_other;
}

@JsonSerializable(includeIfNull: false)
class FamilyMember extends Object with _$FamilyMemberSerializerMixin {
  FamilyMember();

  factory FamilyMember.fromJson(Map<String, dynamic> json) =>
      _$FamilyMemberFromJson(json);

  String member_name;

  @JsonKey(nullable: false)
  List<String> skin_type_ids = [];
  String id;
  int age_started;

  @override
  String toString() => member_name;
}

@JsonSerializable(includeIfNull: false)
class UsedProduct extends Object with _$UsedProductSerializerMixin {
  UsedProduct();

  factory UsedProduct.fromJson(Map<String, dynamic> json) =>
      _$UsedProductFromJson(json);

  String id;
  String name;
  String result;

  /// much better, better, no_change, worse, much_worse
  bool currently_using;

  @override
  String toString() => name;
}

@JsonSerializable(includeIfNull: false)
class UsedTreatment extends Object with _$UsedTreatmentSerializerMixin {
  UsedTreatment();

  factory UsedTreatment.fromJson(Map<String, dynamic> json) =>
      _$UsedTreatmentFromJson(json);

  String id;
  String name;
  String result;
  String times_per_year;

  @override
  String toString() => name;
}
