import 'dart:async';
import 'package:angular/di.dart';
import 'package:firebase/firebase.dart' as fb;

@Injectable()
class DynamicPhraseService {
  DynamicPhraseService();

  void streamLanguage(String language_iso_639_1) {
    if (_childAddedListener != null) _childAddedListener.cancel();
    if (_childChangedListener != null) _childChangedListener.cancel();

    _streamedData = {};

    _q = fb.database().ref('phrases').orderByKey().equalTo(language_iso_639_1);

    _childAddedListener = _q.onChildAdded.listen((qe) {
      _streamedData = qe.snapshot.val();
    });

    _childChangedListener = _q.onChildChanged.listen((qe) {
      _streamedData = qe.snapshot.val();
    });
  }

  /// Return phrases from the stream buffer
  Map<String, String> getPhrases(String category, String key) =>
      _streamedData.containsKey(category) &&
              _streamedData[category].containsKey(key)
          ? _streamedData[category][key]
          : null;

  /// Fetch phrases from the server
  Future<Map<String, Object>> fetchPhrases(
      String language_iso_639_1, String category, String id) async {
    fb.QueryEvent qe;
    qe = await fb
        .database()
        .ref('phrases')
        .child(language_iso_639_1)
        .child(category)
        .child(id)
        .once('value');

    return qe == null ? null : qe.snapshot.val();
  }

  Future setPhrases(String language_iso_639_1, String category, String id,
      Map<String, String> value) async {
    await fb
        .database()
        .ref('phrases')
        .child(language_iso_639_1)
        .child(category)
        .child(id)
        .set(value);
  }

  Map<String, Map<String, Map<String, String>>> _streamedData = {};

  Map<String, Map<String, Map<String, String>>> get streamedData =>
      _streamedData;

  fb.Query _q;
  StreamSubscription<fb.QueryEvent> _childAddedListener;
  StreamSubscription<fb.QueryEvent> _childChangedListener;
}
