part of firestore_service;

@Injectable()
class BookingService extends FirestoreService<Booking> {
  BookingService() : super('bookings');

  /// Find all cached bookings for the specified room id overlapping the specified time
  Booking getByTimeAndRoomId(DateTime time, String room_id) =>
      _cachedModels.values.firstWhere((model) => _overlap(model, room_id, time),
          orElse: () => null);

  @override
  Future<String> push(EditableModel model) async {
    final Booking booking = model;
    if (getByTimeAndRoomId(booking.start_time, booking.room_id) != null)
      throw new Exception(
          'This time (${booking.start_time}) has already been booked');

    booking
      ..cancel_code = _generateCancelCode()
      ..id = await super.push(booking);

    return booking.id;
  }

  @override
  Booking createModelInstance(Map<String, dynamic> data) =>
      new Booking.fromJson(data);

  bool _overlap(EditableModel model, String room_id, DateTime time) {
    final Booking b = model;
    return b.room_id == room_id &&
        ((b.start_time.isAtSameMomentAs(time) ||
            b.start_time.isBefore(time) && b.end_time.isAfter(time)));
  }

  /*
  @override
  Map<String, dynamic> _serialize(Booking model) {
    final data = super._serialize(model);
    data['start_time'] =
        model.start_time == null ? null : timestamp.format(model.start_time);
    data['end_time'] =
        model.end_time == null ? null : timestamp.format(model.end_time);
    return data;
  }

  @override
  Booking _deserialize(Map<String, dynamic> data) {
    try {
      data['start_time'] = DateTime.parse(data['start_time']);
      data['end_time'] = DateTime.parse(data['end_time']);
      return super._deserialize(data);
    } on FormatException catch (e, s) {
      print(e);
      print(s);
      return null;
    }
  }
  */

  String _generateCancelCode() {
    /// generate random string 'abc123'
    final alphaStart = 65;
    final alphaEnd = 90;
    final numStart = 48;
    final numEnd = 57;
    final rnd = new Random();
    final charCodes = new List<int>(6);
    for (var i = 0; i < 3; i++)
      charCodes[i] = rnd.nextInt(alphaEnd - alphaStart) + alphaStart;
    for (var i = 3; i < 6; i++)
      charCodes[i] = rnd.nextInt(numEnd - numStart) + numStart;

    return new String.fromCharCodes(charCodes);
  }

  Booking rebookBuffer;
}
