part of firestore_service;

@Injectable()
class UserService extends FirestoreService<User> {
  UserService() : super('users');

  Future<String> login(String email, String password) async {
    String errorMessage;
    _loading = true;
    try {
      _currentFirebaseUser =
          await fb.auth().signInWithEmailAndPassword(email, password);
      if (_currentFirebaseUser.emailVerified == false)
        throw new Exception('The user is not email verified');

      final matches = await fetchMany(
          whereField: 'email', whereOp: '==', whereValue: email);

      /// If for some reason the user exists in firebase but not in bokain,
      /// or there are more than one user in bokain with that email
      if (matches.isEmpty || matches.length > 1)
        throw new StateError('User integrity corrupt');

      FirestoreService.currentUser = matches.first;
    } on Exception catch (e) {
      errorMessage = e.toString();
      _currentFirebaseUser = null;
      print(errorMessage);
    }

    _loading = false;
    return errorMessage;
  }

  @override
  User createModelInstance(Map<String, dynamic> data) =>
      data == null ? new User() : new User.fromJson(data);

  Future<String> uploadImage(String user_id, String base64) async {
    try {
      _loading = true;
      final parts = base64.split(';base64,');
      final contentType = parts.first.substring('data:'.length);
      final data = parts.last;

      final filename = Uri.encodeFull(user_id);
      final metadata = new fb.UploadMetadata(contentType: contentType);
      await _profileImagesRef
          .child(filename)
          .putString(data, 'base64', metadata)
          .future;
      return (await _profileImagesRef.child(filename).getDownloadURL())
          .toString();
    } on Exception {
      return null;
    } finally {
      _loading = false;
    }
  }

  bool get isLoggedIn => (FirestoreService.currentUser != null);
  String get currentUserId => FirestoreService.currentUser?.id;
  String get currentUserEmail => FirestoreService.currentUser?.email;

  fb.User _currentFirebaseUser;

  final fb.StorageReference _profileImagesRef =
      fb.storage().ref('user-profile-images');
}
